@@ -200,9 +200,9 @@ class Lexicographe:
             if m2:
                 aMorph.append( "-{} : {}".format(m2.group(2), self._formatSuffix(m2.group(2).lower())) )
             # Verbes
-            aVerb = set([ s[1:s.find("/")]  for s in lMorph  if ":V" in s ])
+            aVerb = { s[1:s.find("/")]  for s in lMorph  if ":V" in s }
             return (aMorph, aVerb)
-        except:
+        except (IndexError, TypeError):
             traceback.print_exc()
             return (["#erreur"], None)
 
