@@ -16,7 +16,7 @@ _PATTERNS = {
             r'(?P<HASHTAG>[#@][\w-]+)',
             r'(?P<HTML><\w+.*?>|</\w+ *>)',
             r'(?P<PSEUDOHTML>\[/?\w+\])',
-            r'(?P<HOUR>\d\d?h\d\d\b)',
+            r'(?P<HOUR>\d\d?[h:]\d\d\b)',
             r'(?P<NUM>\d+(?:[.,]\d+))',
             r'(?P<SIGN>[&%‰€$+±=*/<>⩾⩽#|×¥£§¢¬÷@-])',
             r"(?P<WORD>\w+(?:[’'`-]\w+)*)"
@@ -33,7 +33,7 @@ _PATTERNS = {
             r'(?P<PSEUDOHTML>\[/?\w+\])',
             r"(?P<WORD_ELIDED>(?:l|d|n|m|t|s|j|c|ç|lorsqu|puisqu|jusqu|quoiqu|qu)['’`])",
             r'(?P<WORD_ORDINAL>\d+(?:ers?|nds?|es?|des?|ièmes?|èmes?|emes?|ᵉʳˢ?|ⁿᵈˢ?|ᵉˢ?|ᵈᵉˢ?)\b)',
-            r'(?P<HOUR>\d\d?h\d\d\b)',
+            r'(?P<HOUR>\d\d?[h:]\d\d\b)',
             r'(?P<NUM>\d+(?:[.,]\d+|))',
             r'(?P<SIGN>[&%‰€$+±=*/<>⩾⩽#|×¥£¢§¬÷@-])',
             r"(?P<WORD>\w+(?:[’'`-]\w+)*)"
